import sys
from unittest import skip

import generalimport as gi
from generalimport import *

from generalimport.test.funcs import ImportTestCase


class Test(ImportTestCase):
    def test_init_subclass(self):
        """ This one is caught by __call__. """
        generalimport("fakepackage")
        import fakepackage

        with self.assertRaises(MissingDependencyException):
            class X(fakepackage):
                pass








